@@ -37,9 +37,11 @@ impl Handle {
     }
 }
 
-/// Handle to an event structure
+/// Handle to an event structure, guaranteed to be non-null.
+///
+/// If you need to have a nullable event, use `Option<Event>`.
 #[repr(transparent)]
-pub struct Event(*mut c_void);
+pub struct Event(NonNull<c_void>);
 
 impl Event {
     /// Clone this `Event`
@@ -49,7 +51,7 @@ impl Event {
     /// to it are invalidated and the underlying memory is freed by firmware. The caller must ensure
     /// that any clones of a closed `Event` are never used again.
     #[must_use]
-    pub unsafe fn unsafe_clone(&self) -> Self {
+    pub const unsafe fn unsafe_clone(&self) -> Self {
         Self(self.0)
     }
 }
@@ -71,6 +73,7 @@ pub trait Align {
     /// Calculate the offset from `val` necessary to make it aligned,
     /// rounding up. For example, if `val` is 1 and the alignment is 8,
     /// this will return 7. Returns 0 if `val == 0`.
+    #[must_use]
     fn offset_up_to_alignment(val: usize) -> usize {
         assert!(Self::alignment() != 0);
         let r = val % Self::alignment();
@@ -82,6 +85,7 @@ pub trait Align {
     }
 
     /// Round `val` up so that it is aligned.
+    #[must_use]
     fn round_up_to_alignment(val: usize) -> usize {
         val + Self::offset_up_to_alignment(val)
     }
@@ -107,9 +111,17 @@ pub trait Align {
     }
 }
 
+/// Physical memory address. This is always a 64-bit value, regardless
+/// of target platform.
+pub type PhysicalAddress = u64;
+
+/// Virtual memory address. This is always a 64-bit value, regardless
+/// of target platform.
+pub type VirtualAddress = u64;
+
 mod guid;
 pub use self::guid::Guid;
-pub use self::guid::{unsafe_guid, Identify};
+pub use self::guid::Identify;
 
 pub mod chars;
 pub use self::chars::{Char16, Char8};
@@ -119,15 +131,17 @@ mod enums;
 
 mod strs;
 pub use self::strs::{
-    CStr16, CStr8, FromSliceWithNulError, FromStrWithBufError, UnalignedCStr16,
-    UnalignedCStr16Error,
+    CStr16, CStr8, EqStrUntilNul, FromSliceWithNulError, FromStrWithBufError, UnalignedCStr16Error,
 };
 
-#[cfg(feature = "exts")]
+#[cfg(feature = "alloc")]
 mod owned_strs;
-#[cfg(feature = "exts")]
+#[cfg(feature = "alloc")]
 pub use self::owned_strs::{CString16, FromStrError};
 
+mod unaligned_slice;
+pub use unaligned_slice::UnalignedSlice;
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -162,7 +176,7 @@ mod tests {
         assert_eq!(X::round_up_to_alignment(7), 8);
         assert_eq!(X::round_up_to_alignment(8), 8);
 
-        // Get an intentionally mis-aligned buffer.
+        // Get an intentionally misaligned buffer.
         let mut buffer = [0u8; 16];
         let mut buffer = &mut buffer[..];
         if (buffer.as_ptr() as usize) % X::alignment() == 0 {
