@@ -1,8 +1,6 @@
 //! Graphics output protocol.
 //!
 //! The UEFI GOP is meant to replace existing [VGA][vga] hardware interfaces.
-//! It can be used in the boot environment as well as at runtime,
-//! until a high-performance driver is loaded by the OS.
 //!
 //! The GOP provides access to a hardware frame buffer and allows UEFI apps
 //! to draw directly to the graphics output device.
@@ -11,8 +9,6 @@
 //! to exist and be used on the system. There is a GOP implementation for every
 //! unique GPU in the system which supports UEFI.
 //!
-//! This protocol _can_ be used after boot services are exited.
-//!
 //! [vga]: https://en.wikipedia.org/wiki/Video_Graphics_Array
 //!
 //! # Definitions
@@ -54,8 +50,9 @@
 //! You will have to implement your own double buffering if you want to
 //! avoid tearing with animations.
 
-use crate::proto::Protocol;
-use crate::{unsafe_guid, Result, Status};
+use crate::proto::unsafe_protocol;
+use crate::util::usize_from_u32;
+use crate::{Result, Status};
 use core::marker::PhantomData;
 use core::mem;
 use core::ptr;
@@ -65,9 +62,8 @@ use core::ptr;
 /// The GOP can be used to set the properties of the frame buffer,
 /// and also allows the app to access the in-memory buffer.
 #[repr(C)]
-#[unsafe_guid("9042a9de-23dc-4a38-96fb-7aded080516a")]
-#[derive(Protocol)]
-pub struct GraphicsOutput<'boot> {
+#[unsafe_protocol("9042a9de-23dc-4a38-96fb-7aded080516a")]
+pub struct GraphicsOutput {
     query_mode: extern "efiapi" fn(
         &GraphicsOutput,
         mode: u32,
@@ -88,10 +84,10 @@ pub struct GraphicsOutput<'boot> {
         height: usize,
         stride: usize,
     ) -> Status,
-    mode: &'boot ModeData<'boot>,
+    mode: *const ModeData,
 }
 
-impl<'boot> GraphicsOutput<'boot> {
+impl GraphicsOutput {
     /// Returns information for an available graphics mode that the graphics
     /// device and the set of active video output devices supports.
     pub fn query_mode(&self, index: u32) -> Result<Mode> {
@@ -109,11 +105,12 @@ impl<'boot> GraphicsOutput<'boot> {
     }
 
     /// Returns information about all available graphics modes.
-    pub fn modes(&'_ self) -> impl ExactSizeIterator<Item = Mode> + '_ {
+    #[must_use]
+    pub fn modes(&self) -> ModeIter {
         ModeIter {
             gop: self,
             current: 0,
-            max: self.mode.max_mode,
+            max: self.mode().max_mode,
         }
     }
 
@@ -294,18 +291,19 @@ impl<'boot> GraphicsOutput<'boot> {
     }
 
     /// Returns the frame buffer information for the current mode.
-    pub fn current_mode_info(&self) -> ModeInfo {
-        *self.mode.info
+    #[must_use]
+    pub const fn current_mode_info(&self) -> ModeInfo {
+        *self.mode().info()
     }
 
     /// Access the frame buffer directly
     pub fn frame_buffer(&mut self) -> FrameBuffer {
         assert!(
-            self.mode.info.format != PixelFormat::BltOnly,
+            self.mode().info().format != PixelFormat::BltOnly,
             "Cannot access the framebuffer in a Blt-only mode"
         );
-        let base = self.mode.fb_address as *mut u8;
-        let size = self.mode.fb_size;
+        let base = self.mode().fb_address as *mut u8;
+        let size = self.mode().fb_size;
 
         FrameBuffer {
             base,
@@ -313,16 +311,20 @@ impl<'boot> GraphicsOutput<'boot> {
             _lifetime: PhantomData,
         }
     }
+
+    const fn mode(&self) -> &ModeData {
+        unsafe { &*self.mode }
+    }
 }
 
 #[repr(C)]
-struct ModeData<'info> {
+struct ModeData {
     // Number of modes which the GOP supports.
     max_mode: u32,
     // Current mode.
     mode: u32,
     // Information about the current mode.
-    info: &'info ModeInfo,
+    info: *const ModeInfo,
     // Size of the above structure.
     info_sz: usize,
     // Physical address of the frame buffer.
@@ -331,6 +333,12 @@ struct ModeData<'info> {
     fb_size: usize,
 }
 
+impl ModeData {
+    const fn info(&self) -> &ModeInfo {
+        unsafe { &*self.info }
+    }
+}
+
 /// Represents the format of the pixels in a frame buffer.
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 #[repr(u32)]
@@ -377,12 +385,14 @@ impl Mode {
     /// The size of the info structure in bytes.
     ///
     /// Newer versions of the spec might add extra information, in a backwards compatible way.
-    pub fn info_size(&self) -> usize {
+    #[must_use]
+    pub const fn info_size(&self) -> usize {
         self.info_sz
     }
 
     /// Returns a reference to the mode info structure.
-    pub fn info(&self) -> &ModeInfo {
+    #[must_use]
+    pub const fn info(&self) -> &ModeInfo {
         &self.info
     }
 }
@@ -404,17 +414,20 @@ impl ModeInfo {
     /// Returns the (horizontal, vertical) resolution.
     ///
     /// On desktop monitors, this usually means (width, height).
-    pub fn resolution(&self) -> (usize, usize) {
-        (self.hor_res as usize, self.ver_res as usize)
+    #[must_use]
+    pub const fn resolution(&self) -> (usize, usize) {
+        (usize_from_u32(self.hor_res), usize_from_u32(self.ver_res))
     }
 
     /// Returns the format of the frame buffer.
-    pub fn pixel_format(&self) -> PixelFormat {
+    #[must_use]
+    pub const fn pixel_format(&self) -> PixelFormat {
         self.format
     }
 
     /// Returns the bitmask of the custom pixel format, if available.
-    pub fn pixel_bitmask(&self) -> Option<PixelBitmask> {
+    #[must_use]
+    pub const fn pixel_bitmask(&self) -> Option<PixelBitmask> {
         match self.format {
             PixelFormat::Bitmask => Some(self.mask),
             _ => None,
@@ -425,14 +438,15 @@ impl ModeInfo {
     ///
     /// Due to performance reasons, the stride might not be equal to the width,
     /// instead the stride might be bigger for better alignment.
-    pub fn stride(&self) -> usize {
-        self.stride as usize
+    #[must_use]
+    pub const fn stride(&self) -> usize {
+        usize_from_u32(self.stride)
     }
 }
 
-/// Iterator for graphics modes.
-struct ModeIter<'gop> {
-    gop: &'gop GraphicsOutput<'gop>,
+/// Iterator for [`Mode`]s of the [`GraphicsOutput`] protocol.
+pub struct ModeIter<'gop> {
+    gop: &'gop GraphicsOutput,
     current: u32,
     max: u32,
 }
@@ -475,7 +489,8 @@ pub struct BltPixel {
 
 impl BltPixel {
     /// Create a new pixel from RGB values.
-    pub fn new(red: u8, green: u8, blue: u8) -> Self {
+    #[must_use]
+    pub const fn new(red: u8, green: u8, blue: u8) -> Self {
         Self {
             red,
             green,
@@ -577,12 +592,16 @@ impl<'gop> FrameBuffer<'gop> {
     /// - Keep memory accesses in bound
     /// - Use volatile reads and writes
     /// - Make sure that the pointer does not outlive the FrameBuffer
+    ///
+    /// On some implementations this framebuffer pointer can be used after
+    /// exiting boot services, but that is not guaranteed by the UEFI Specification.
     pub fn as_mut_ptr(&mut self) -> *mut u8 {
         self.base
     }
 
     /// Query the framebuffer size in bytes
-    pub fn size(&self) -> usize {
+    #[must_use]
+    pub const fn size(&self) -> usize {
         self.size
     }
 
@@ -607,6 +626,7 @@ impl<'gop> FrameBuffer<'gop> {
     /// - You must honor the pixel format and stride specified by the mode info
     /// - There is no bound checking on memory accesses in release mode
     #[inline]
+    #[must_use]
     pub unsafe fn read_byte(&self, index: usize) -> u8 {
         debug_assert!(index < self.size, "Frame buffer accessed out of bounds");
         self.base.add(index).read_volatile()
@@ -621,7 +641,7 @@ impl<'gop> FrameBuffer<'gop> {
     /// # Safety
     ///
     /// This operation is unsafe because...
-    /// - It is your reponsibility to make sure that the value type makes sense
+    /// - It is your responsibility to make sure that the value type makes sense
     /// - You must honor the pixel format and stride specified by the mode info
     /// - There is no bound checking on memory accesses in release mode
     #[inline]
@@ -643,10 +663,11 @@ impl<'gop> FrameBuffer<'gop> {
     /// # Safety
     ///
     /// This operation is unsafe because...
-    /// - It is your reponsibility to make sure that the value type makes sense
+    /// - It is your responsibility to make sure that the value type makes sense
     /// - You must honor the pixel format and stride specified by the mode info
     /// - There is no bound checking on memory accesses in release mode
     #[inline]
+    #[must_use]
     pub unsafe fn read_value<T>(&self, index: usize) -> T {
         debug_assert!(
             index.saturating_add(mem::size_of::<T>()) <= self.size,
