@@ -1,5 +1,5 @@
-use crate::proto::Protocol;
-use crate::{unsafe_guid, CStr16, Char16, Result, ResultExt, Status};
+use crate::proto::unsafe_protocol;
+use crate::{CStr16, Char16, Result, ResultExt, Status};
 use core::fmt;
 use core::fmt::{Debug, Formatter};
 
@@ -21,9 +21,8 @@ use core::fmt::{Debug, Formatter};
 /// [`SystemTable::stderr`]: crate::table::SystemTable::stderr
 /// [`BootServices`]: crate::table::boot::BootServices#accessing-protocols
 #[repr(C)]
-#[unsafe_guid("387477c2-69c7-11d2-8e39-00a0c969723b")]
-#[derive(Protocol)]
-pub struct Output<'boot> {
+#[unsafe_protocol("387477c2-69c7-11d2-8e39-00a0c969723b")]
+pub struct Output {
     reset: extern "efiapi" fn(this: &Output, extended: bool) -> Status,
     output_string: unsafe extern "efiapi" fn(this: &Output, string: *const Char16) -> Status,
     test_string: unsafe extern "efiapi" fn(this: &Output, string: *const Char16) -> Status,
@@ -38,10 +37,10 @@ pub struct Output<'boot> {
     clear_screen: extern "efiapi" fn(this: &mut Output) -> Status,
     set_cursor_position: extern "efiapi" fn(this: &mut Output, column: usize, row: usize) -> Status,
     enable_cursor: extern "efiapi" fn(this: &mut Output, visible: bool) -> Status,
-    data: &'boot OutputData,
+    data: *const OutputData,
 }
 
-impl<'boot> Output<'boot> {
+impl Output {
     /// Resets and clears the text output device hardware.
     pub fn reset(&mut self, extended: bool) -> Result {
         (self.reset)(self, extended).into()
@@ -86,8 +85,8 @@ impl<'boot> Output<'boot> {
 
     /// Returns an iterator of all supported text modes.
     // TODO: Bring back impl Trait once the story around bounds improves
-    pub fn modes<'out>(&'out mut self) -> OutputModeIter<'out, 'boot> {
-        let max = self.data.max_mode as usize;
+    pub fn modes(&mut self) -> OutputModeIter<'_> {
+        let max = self.data().max_mode as usize;
         OutputModeIter {
             output: self,
             current: 0,
@@ -110,9 +109,9 @@ impl<'boot> Output<'boot> {
         (self.query_mode)(self, index, &mut columns, &mut rows).into_with_val(|| (columns, rows))
     }
 
-    /// Returns the the current text mode.
+    /// Returns the current text mode.
     pub fn current_mode(&self) -> Result<Option<OutputMode>> {
-        match self.data.mode {
+        match self.data().mode {
             -1 => Ok(None),
             n if n >= 0 => {
                 let index = n as usize;
@@ -129,8 +128,9 @@ impl<'boot> Output<'boot> {
     }
 
     /// Returns whether the cursor is currently shown or not.
-    pub fn cursor_visible(&self) -> bool {
-        self.data.cursor_visible
+    #[must_use]
+    pub const fn cursor_visible(&self) -> bool {
+        self.data().cursor_visible
     }
 
     /// Make the cursor visible or invisible.
@@ -142,9 +142,10 @@ impl<'boot> Output<'boot> {
     }
 
     /// Returns the column and row of the cursor.
-    pub fn cursor_position(&self) -> (usize, usize) {
-        let column = self.data.cursor_column;
-        let row = self.data.cursor_row;
+    #[must_use]
+    pub const fn cursor_position(&self) -> (usize, usize) {
+        let column = self.data().cursor_column;
+        let row = self.data().cursor_row;
         (column as usize, row as usize)
     }
 
@@ -168,9 +169,15 @@ impl<'boot> Output<'boot> {
         let attr = ((bgc & 0x7) << 4) | (fgc & 0xF);
         (self.set_attribute)(self, attr).into()
     }
+
+    /// Get a reference to `OutputData`. The lifetime of the reference is tied
+    /// to `self`.
+    const fn data(&self) -> &OutputData {
+        unsafe { &*self.data }
+    }
 }
 
-impl<'boot> fmt::Write for Output<'boot> {
+impl fmt::Write for Output {
     fn write_str(&mut self, s: &str) -> fmt::Result {
         // Allocate a small buffer on the stack.
         const BUF_SIZE: usize = 128;
@@ -221,7 +228,7 @@ impl<'boot> fmt::Write for Output<'boot> {
     }
 }
 
-impl<'boot> Debug for Output<'boot> {
+impl Debug for Output {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         f.debug_struct("Output")
             .field("reset (fn ptr)", &(self.reset as *const u64))
@@ -260,31 +267,34 @@ pub struct OutputMode {
 impl OutputMode {
     /// Returns the index of this mode.
     #[inline]
-    pub fn index(&self) -> usize {
+    #[must_use]
+    pub const fn index(&self) -> usize {
         self.index
     }
 
     /// Returns the width in columns.
     #[inline]
-    pub fn columns(&self) -> usize {
+    #[must_use]
+    pub const fn columns(&self) -> usize {
         self.dims.0
     }
 
     /// Returns the height in rows.
     #[inline]
-    pub fn rows(&self) -> usize {
+    #[must_use]
+    pub const fn rows(&self) -> usize {
         self.dims.1
     }
 }
 
 /// An iterator of the text modes (possibly) supported by a device.
-pub struct OutputModeIter<'out, 'boot: 'out> {
-    output: &'out mut Output<'boot>,
+pub struct OutputModeIter<'out> {
+    output: &'out mut Output,
     current: usize,
     max: usize,
 }
 
-impl<'out, 'boot> Iterator for OutputModeIter<'out, 'boot> {
+impl<'out> Iterator for OutputModeIter<'out> {
     type Item = OutputMode;
 
     fn next(&mut self) -> Option<Self::Item> {
