@@ -1,7 +1,7 @@
 //! Device Path protocol
 //!
 //! A UEFI device path is a very flexible structure for encoding a
-//! programatic path such as a hard drive or console.
+//! programmatic path such as a hard drive or console.
 //!
 //! A device path is made up of a packed list of variable-length nodes of
 //! various types. The entire device path is terminated with an
@@ -39,10 +39,17 @@
 //!   node's [`device_type`] and [`sub_type`] must be examined to
 //!   determine what type of data it contains.
 //!
-//!   Specific node types have their own structures, but only a few are
-//!   currently implemented:
-//!   * [`AcpiDevicePath`]
-//!   * [`FilePathMediaDevicePath`]
+//!   Specific node types have their own structures in these submodules:
+//!   * [`acpi`]
+//!   * [`bios_boot_spec`]
+//!   * [`end`]
+//!   * [`hardware`]
+//!   * [`media`]
+//!   * [`messaging`]
+//!
+//! * [`DevicePathNodeEnum`] contains variants for references to each
+//!   type of node. Call [`DevicePathNode::as_enum`] to convert from a
+//!   [`DevicePathNode`] reference to a `DevicePathNodeEnum`.
 //!
 //! * [`DevicePathHeader`] is a header present at the start of every
 //!   node. It describes the type of node as well as the node's size.
@@ -62,17 +69,24 @@
 //!
 //! [`END_ENTIRE`]: DeviceSubType::END_ENTIRE
 //! [`END_INSTANCE`]: DeviceSubType::END_INSTANCE
+//! [`Protocol`]: crate::proto::Protocol
 //! [`device_type`]: DevicePathNode::device_type
 //! [`sub_type`]: DevicePathNode::sub_type
 
+pub mod build;
 pub mod text;
 
-use crate::data_types::UnalignedCStr16;
-use crate::proto::{Protocol, ProtocolPointer};
-use crate::unsafe_guid;
+mod device_path_gen;
+pub use device_path_gen::{
+    acpi, bios_boot_spec, end, hardware, media, messaging, DevicePathNodeEnum,
+};
+
+use crate::proto::{unsafe_protocol, ProtocolPointer};
 use core::ffi::c_void;
+use core::fmt::{self, Debug, Formatter};
 use core::marker::{PhantomData, PhantomPinned};
-use core::{mem, ptr};
+use core::mem;
+use ptr_meta::Pointee;
 
 /// Opaque type that should be used to represent a pointer to a
 /// [`DevicePath`] or [`DevicePathNode`] in foreign function interfaces. This
@@ -106,7 +120,7 @@ pub struct DevicePathHeader {
 /// See the [module-level documentation] for more details.
 ///
 /// [module-level documentation]: crate::proto::device_path
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Eq, Pointee)]
 #[repr(C, packed)]
 pub struct DevicePathNode {
     header: DevicePathHeader,
@@ -121,61 +135,70 @@ impl DevicePathNode {
     /// The input pointer must point to valid data. That data must
     /// remain valid for the lifetime `'a`, and cannot be mutated during
     /// that lifetime.
+    #[must_use]
     pub unsafe fn from_ffi_ptr<'a>(ptr: *const FfiDevicePath) -> &'a DevicePathNode {
         let header = *ptr.cast::<DevicePathHeader>();
 
         let data_len = usize::from(header.length) - mem::size_of::<DevicePathHeader>();
-        &*ptr::from_raw_parts(ptr.cast(), data_len)
+        &*ptr_meta::from_raw_parts(ptr.cast(), data_len)
     }
 
     /// Cast to a [`FfiDevicePath`] pointer.
-    pub fn as_ffi_ptr(&self) -> *const FfiDevicePath {
+    #[must_use]
+    pub const fn as_ffi_ptr(&self) -> *const FfiDevicePath {
         let ptr: *const Self = self;
         ptr.cast::<FfiDevicePath>()
     }
 
     /// Type of device
-    pub fn device_type(&self) -> DeviceType {
+    #[must_use]
+    pub const fn device_type(&self) -> DeviceType {
         self.header.device_type
     }
 
     /// Sub type of device
-    pub fn sub_type(&self) -> DeviceSubType {
+    #[must_use]
+    pub const fn sub_type(&self) -> DeviceSubType {
         self.header.sub_type
     }
 
     /// Tuple of the node's type and subtype.
-    pub fn full_type(&self) -> (DeviceType, DeviceSubType) {
+    #[must_use]
+    pub const fn full_type(&self) -> (DeviceType, DeviceSubType) {
         (self.header.device_type, self.header.sub_type)
     }
 
     /// Size (in bytes) of the full [`DevicePathNode`], including the header.
-    pub fn length(&self) -> u16 {
+    #[must_use]
+    pub const fn length(&self) -> u16 {
         self.header.length
     }
 
     /// True if this node ends an entire [`DevicePath`].
+    #[must_use]
     pub fn is_end_entire(&self) -> bool {
         self.full_type() == (DeviceType::END, DeviceSubType::END_ENTIRE)
     }
 
-    /// Convert to a [`FilePathMediaDevicePath`]. Returns `None` if the
-    /// node is not of the appropriate type.
-    pub fn as_file_path_media_device_path(&self) -> Option<&FilePathMediaDevicePath> {
-        if self.full_type() == (DeviceType::MEDIA, DeviceSubType::MEDIA_FILE_PATH) {
-            // Get the length of the `path_name` field.
-            let path_name_size_in_bytes = usize::from(self.header.length) - 4;
-            assert!(path_name_size_in_bytes % mem::size_of::<u16>() == 0);
-            let path_name_len = path_name_size_in_bytes / mem::size_of::<u16>();
+    /// Convert from a generic [`DevicePathNode`] reference to an enum
+    /// of more specific node types.
+    pub fn as_enum(&self) -> Result<DevicePathNodeEnum, NodeConversionError> {
+        DevicePathNodeEnum::try_from(self)
+    }
+}
 
-            // Convert the `self` pointer to `FilePathMediaDevicePath`,
-            // which is a DST as it ends in a slice.
-            let p = self as *const Self;
-            let p: *const FilePathMediaDevicePath = ptr::from_raw_parts(p.cast(), path_name_len);
-            Some(unsafe { &*p })
-        } else {
-            None
-        }
+impl Debug for DevicePathNode {
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+        f.debug_struct("DevicePathNode")
+            .field("header", &self.header)
+            .field("data", &&self.data)
+            .finish()
+    }
+}
+
+impl PartialEq for DevicePathNode {
+    fn eq(&self, other: &Self) -> bool {
+        self.header == other.header && self.data == other.data
     }
 }
 
@@ -189,7 +212,7 @@ impl DevicePathNode {
 /// [`END_INSTANCE`]: DeviceSubType::END_INSTANCE
 /// [module-level documentation]: crate::proto::device_path
 #[repr(C, packed)]
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Eq, Pointee)]
 pub struct DevicePathInstance {
     data: [u8],
 }
@@ -200,7 +223,8 @@ impl DevicePathInstance {
     /// reached.
     ///
     /// [`DevicePathNodes`]: DevicePathNode
-    pub fn node_iter(&self) -> DevicePathNodeIterator {
+    #[must_use]
+    pub const fn node_iter(&self) -> DevicePathNodeIterator {
         DevicePathNodeIterator {
             nodes: &self.data,
             stop_condition: StopCondition::AnyEndNode,
@@ -208,6 +232,20 @@ impl DevicePathInstance {
     }
 }
 
+impl Debug for DevicePathInstance {
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+        f.debug_struct("DevicePathInstance")
+            .field("data", &&self.data)
+            .finish()
+    }
+}
+
+impl PartialEq for DevicePathInstance {
+    fn eq(&self, other: &Self) -> bool {
+        self.data == other.data
+    }
+}
+
 /// Device path protocol.
 ///
 /// A device path contains one or more device path instances made of up
@@ -218,19 +256,19 @@ impl DevicePathInstance {
 /// [module-level documentation]: crate::proto::device_path
 /// [`END_ENTIRE`]: DeviceSubType::END_ENTIRE
 #[repr(C, packed)]
-#[unsafe_guid("09576e91-6d3f-11d2-8e39-00a0c969723b")]
-#[derive(Debug, Eq, PartialEq, Protocol)]
+#[unsafe_protocol("09576e91-6d3f-11d2-8e39-00a0c969723b")]
+#[derive(Eq, Pointee)]
 pub struct DevicePath {
     data: [u8],
 }
 
 impl ProtocolPointer for DevicePath {
     unsafe fn ptr_from_ffi(ptr: *const c_void) -> *const Self {
-        ptr::from_raw_parts(ptr.cast(), Self::size_in_bytes_from_ptr(ptr))
+        ptr_meta::from_raw_parts(ptr.cast(), Self::size_in_bytes_from_ptr(ptr))
     }
 
     unsafe fn mut_ptr_from_ffi(ptr: *mut c_void) -> *mut Self {
-        ptr::from_raw_parts_mut(ptr.cast(), Self::size_in_bytes_from_ptr(ptr))
+        ptr_meta::from_raw_parts_mut(ptr.cast(), Self::size_in_bytes_from_ptr(ptr))
     }
 }
 
@@ -261,18 +299,21 @@ impl DevicePath {
     /// The input pointer must point to valid data. That data must
     /// remain valid for the lifetime `'a`, and cannot be mutated during
     /// that lifetime.
+    #[must_use]
     pub unsafe fn from_ffi_ptr<'a>(ptr: *const FfiDevicePath) -> &'a DevicePath {
         &*Self::ptr_from_ffi(ptr.cast::<c_void>())
     }
 
     /// Cast to a [`FfiDevicePath`] pointer.
-    pub fn as_ffi_ptr(&self) -> *const FfiDevicePath {
+    #[must_use]
+    pub const fn as_ffi_ptr(&self) -> *const FfiDevicePath {
         let p = self as *const Self;
         p.cast()
     }
 
     /// Get an iterator over the [`DevicePathInstance`]s in this path.
-    pub fn instance_iter(&self) -> DevicePathInstanceIterator {
+    #[must_use]
+    pub const fn instance_iter(&self) -> DevicePathInstanceIterator {
         DevicePathInstanceIterator {
             remaining_path: Some(self),
         }
@@ -282,7 +323,8 @@ impl DevicePath {
     /// `self`. Iteration ends when a path is reached where
     /// [`is_end_entire`][DevicePathNode::is_end_entire] is true. That ending
     /// path is not returned by the iterator.
-    pub fn node_iter(&self) -> DevicePathNodeIterator {
+    #[must_use]
+    pub const fn node_iter(&self) -> DevicePathNodeIterator {
         DevicePathNodeIterator {
             nodes: &self.data,
             stop_condition: StopCondition::EndEntireNode,
@@ -290,6 +332,20 @@ impl DevicePath {
     }
 }
 
+impl Debug for DevicePath {
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+        f.debug_struct("DevicePath")
+            .field("data", &&self.data)
+            .finish()
+    }
+}
+
+impl PartialEq for DevicePath {
+    fn eq(&self, other: &Self) -> bool {
+        self.data == other.data
+    }
+}
+
 /// Iterator over the [`DevicePathInstance`]s in a [`DevicePath`].
 ///
 /// This struct is returned by [`DevicePath::instance_iter`].
@@ -325,7 +381,7 @@ impl<'a> Iterator for DevicePathInstanceIterator<'a> {
             self.remaining_path = None;
         } else {
             self.remaining_path = unsafe {
-                Some(&*ptr::from_raw_parts(
+                Some(&*ptr_meta::from_raw_parts(
                     rest.as_ptr().cast::<()>(),
                     rest.len(),
                 ))
@@ -333,7 +389,7 @@ impl<'a> Iterator for DevicePathInstanceIterator<'a> {
         }
 
         unsafe {
-            Some(&*ptr::from_raw_parts(
+            Some(&*ptr_meta::from_raw_parts(
                 head.as_ptr().cast::<()>(),
                 head.len(),
             ))
@@ -431,109 +487,113 @@ pub struct DeviceSubType(pub u8);
 
 impl DeviceSubType {
     /// PCI Device Path.
-    pub const HARDWARE_PCI: DeviceSubType = DeviceSubType(0x01);
+    pub const HARDWARE_PCI: DeviceSubType = DeviceSubType(1);
     /// PCCARD Device Path.
-    pub const HARDWARE_PCCARD: DeviceSubType = DeviceSubType(0x02);
+    pub const HARDWARE_PCCARD: DeviceSubType = DeviceSubType(2);
     /// Memory-mapped Device Path.
-    pub const HARDWARE_MEMORY_MAPPED: DeviceSubType = DeviceSubType(0x03);
+    pub const HARDWARE_MEMORY_MAPPED: DeviceSubType = DeviceSubType(3);
     /// Vendor-Defined Device Path.
-    pub const HARDWARE_VENDOR: DeviceSubType = DeviceSubType(0x04);
+    pub const HARDWARE_VENDOR: DeviceSubType = DeviceSubType(4);
     /// Controller Device Path.
-    pub const HARDWARE_CONTROLLER: DeviceSubType = DeviceSubType(0x05);
+    pub const HARDWARE_CONTROLLER: DeviceSubType = DeviceSubType(5);
     /// BMC Device Path.
-    pub const HARDWARE_BMC: DeviceSubType = DeviceSubType(0x06);
+    pub const HARDWARE_BMC: DeviceSubType = DeviceSubType(6);
 
     /// ACPI Device Path.
-    pub const ACPI: DeviceSubType = DeviceSubType(0x01);
+    pub const ACPI: DeviceSubType = DeviceSubType(1);
     /// Expanded ACPI Device Path.
-    pub const ACPI_EXPANDED: DeviceSubType = DeviceSubType(0x02);
+    pub const ACPI_EXPANDED: DeviceSubType = DeviceSubType(2);
     /// ACPI _ADR Device Path.
-    pub const ACPI_ADR: DeviceSubType = DeviceSubType(0x03);
+    pub const ACPI_ADR: DeviceSubType = DeviceSubType(3);
     /// NVDIMM Device Path.
-    pub const ACPI_NVDIMM: DeviceSubType = DeviceSubType(0x04);
+    pub const ACPI_NVDIMM: DeviceSubType = DeviceSubType(4);
 
     /// ATAPI Device Path.
-    pub const MESSAGING_ATAPI: DeviceSubType = DeviceSubType(0x01);
+    pub const MESSAGING_ATAPI: DeviceSubType = DeviceSubType(1);
     /// SCSI Device Path.
-    pub const MESSAGING_SCSI: DeviceSubType = DeviceSubType(0x02);
+    pub const MESSAGING_SCSI: DeviceSubType = DeviceSubType(2);
     /// Fibre Channel Device Path.
-    pub const MESSAGING_FIBRE_CHANNEL: DeviceSubType = DeviceSubType(0x03);
+    pub const MESSAGING_FIBRE_CHANNEL: DeviceSubType = DeviceSubType(3);
     /// 1394 Device Path.
-    pub const MESSAGING_1394: DeviceSubType = DeviceSubType(0x04);
+    pub const MESSAGING_1394: DeviceSubType = DeviceSubType(4);
     /// USB Device Path.
-    pub const MESSAGING_USB: DeviceSubType = DeviceSubType(0x05);
+    pub const MESSAGING_USB: DeviceSubType = DeviceSubType(5);
     /// I2O Device Path.
-    pub const MESSAGING_I2O: DeviceSubType = DeviceSubType(0x06);
+    pub const MESSAGING_I2O: DeviceSubType = DeviceSubType(6);
     /// Infiniband Device Path.
-    pub const MESSAGING_INFINIBAND: DeviceSubType = DeviceSubType(0x09);
+    pub const MESSAGING_INFINIBAND: DeviceSubType = DeviceSubType(9);
     /// Vendor-Defined Device Path.
-    pub const MESSAGING_VENDOR: DeviceSubType = DeviceSubType(0x0a);
+    pub const MESSAGING_VENDOR: DeviceSubType = DeviceSubType(10);
     /// MAC Address Device Path.
-    pub const MESSAGING_MAC_ADDRESS: DeviceSubType = DeviceSubType(0x0b);
+    pub const MESSAGING_MAC_ADDRESS: DeviceSubType = DeviceSubType(11);
     /// IPV4 Device Path.
-    pub const MESSAGING_IPV4: DeviceSubType = DeviceSubType(0x0c);
+    pub const MESSAGING_IPV4: DeviceSubType = DeviceSubType(12);
     /// IPV6 Device Path.
-    pub const MESSAGING_IPV6: DeviceSubType = DeviceSubType(0x0d);
+    pub const MESSAGING_IPV6: DeviceSubType = DeviceSubType(13);
     /// UART Device Path.
-    pub const MESSAGING_UART: DeviceSubType = DeviceSubType(0x0e);
+    pub const MESSAGING_UART: DeviceSubType = DeviceSubType(14);
     /// USB Class Device Path.
-    pub const MESSAGING_USB_CLASS: DeviceSubType = DeviceSubType(0x0f);
+    pub const MESSAGING_USB_CLASS: DeviceSubType = DeviceSubType(15);
     /// USB WWID Device Path.
-    pub const MESSAGING_USB_WWID: DeviceSubType = DeviceSubType(0x10);
+    pub const MESSAGING_USB_WWID: DeviceSubType = DeviceSubType(16);
     /// Device Logical Unit.
-    pub const MESSAGING_DEVICE_LOGICAL_UNIT: DeviceSubType = DeviceSubType(0x11);
+    pub const MESSAGING_DEVICE_LOGICAL_UNIT: DeviceSubType = DeviceSubType(17);
     /// SATA Device Path.
-    pub const MESSAGING_SATA: DeviceSubType = DeviceSubType(0x12);
+    pub const MESSAGING_SATA: DeviceSubType = DeviceSubType(18);
     /// iSCSI Device Path node (base information).
-    pub const MESSAGING_ISCSI: DeviceSubType = DeviceSubType(0x13);
+    pub const MESSAGING_ISCSI: DeviceSubType = DeviceSubType(19);
     /// VLAN Device Path node.
-    pub const MESSAGING_VLAN: DeviceSubType = DeviceSubType(0x14);
+    pub const MESSAGING_VLAN: DeviceSubType = DeviceSubType(20);
     /// Fibre Channel Ex Device Path.
-    pub const MESSAGING_FIBRE_CHANNEL_EX: DeviceSubType = DeviceSubType(0x15);
+    pub const MESSAGING_FIBRE_CHANNEL_EX: DeviceSubType = DeviceSubType(21);
     /// Serial Attached SCSI (SAS) Ex Device Path.
-    pub const MESSAGING_SCSI_SAS_EX: DeviceSubType = DeviceSubType(0x16);
+    pub const MESSAGING_SCSI_SAS_EX: DeviceSubType = DeviceSubType(22);
     /// NVM Express Namespace Device Path.
-    pub const MESSAGING_NVME_NAMESPACE: DeviceSubType = DeviceSubType(0x17);
+    pub const MESSAGING_NVME_NAMESPACE: DeviceSubType = DeviceSubType(23);
     /// Uniform Resource Identifiers (URI) Device Path.
-    pub const MESSAGING_URI: DeviceSubType = DeviceSubType(0x18);
+    pub const MESSAGING_URI: DeviceSubType = DeviceSubType(24);
     /// UFS Device Path.
-    pub const MESSAGING_UFS: DeviceSubType = DeviceSubType(0x19);
+    pub const MESSAGING_UFS: DeviceSubType = DeviceSubType(25);
     /// SD (Secure Digital) Device Path.
-    pub const MESSAGING_SD: DeviceSubType = DeviceSubType(0x1a);
+    pub const MESSAGING_SD: DeviceSubType = DeviceSubType(26);
     /// Bluetooth Device Path.
-    pub const MESSAGING_BLUETOOTH: DeviceSubType = DeviceSubType(0x1b);
+    pub const MESSAGING_BLUETOOTH: DeviceSubType = DeviceSubType(27);
     /// Wi-Fi Device Path.
-    pub const MESSAGING_WIFI: DeviceSubType = DeviceSubType(0x1c);
+    pub const MESSAGING_WIFI: DeviceSubType = DeviceSubType(28);
     /// eMMC (Embedded Multi-Media Card) Device Path.
-    pub const MESSAGING_EMMC: DeviceSubType = DeviceSubType(0x1d);
+    pub const MESSAGING_EMMC: DeviceSubType = DeviceSubType(29);
     /// BluetoothLE Device Path.
-    pub const MESSAGING_BLUETOOTH_LE: DeviceSubType = DeviceSubType(0x1e);
+    pub const MESSAGING_BLUETOOTH_LE: DeviceSubType = DeviceSubType(30);
     /// DNS Device Path.
-    pub const MESSAGING_DNS: DeviceSubType = DeviceSubType(0x1f);
+    pub const MESSAGING_DNS: DeviceSubType = DeviceSubType(31);
     /// NVDIMM Namespace Device Path.
-    pub const MESSAGING_NVDIMM_NAMESPACE: DeviceSubType = DeviceSubType(0x20);
+    pub const MESSAGING_NVDIMM_NAMESPACE: DeviceSubType = DeviceSubType(32);
+    /// REST Service Device Path.
+    pub const MESSAGING_REST_SERVICE: DeviceSubType = DeviceSubType(33);
+    /// NVME over Fabric (NVMe-oF) Namespace Device Path.
+    pub const MESSAGING_NVME_OF_NAMESPACE: DeviceSubType = DeviceSubType(34);
 
     /// Hard Drive Media Device Path.
-    pub const MEDIA_HARD_DRIVE: DeviceSubType = DeviceSubType(0x01);
+    pub const MEDIA_HARD_DRIVE: DeviceSubType = DeviceSubType(1);
     /// CD-ROM Media Device Path.
-    pub const MEDIA_CD_ROM: DeviceSubType = DeviceSubType(0x02);
+    pub const MEDIA_CD_ROM: DeviceSubType = DeviceSubType(2);
     /// Vendor-Defined Media Device Path.
-    pub const MEDIA_VENDOR: DeviceSubType = DeviceSubType(0x03);
+    pub const MEDIA_VENDOR: DeviceSubType = DeviceSubType(3);
     /// File Path Media Device Path.
-    pub const MEDIA_FILE_PATH: DeviceSubType = DeviceSubType(0x04);
+    pub const MEDIA_FILE_PATH: DeviceSubType = DeviceSubType(4);
     /// Media Protocol Device Path.
-    pub const MEDIA_PROTOCOL: DeviceSubType = DeviceSubType(0x05);
+    pub const MEDIA_PROTOCOL: DeviceSubType = DeviceSubType(5);
     /// PIWG Firmware File.
-    pub const MEDIA_PIWG_FIRMWARE_FILE: DeviceSubType = DeviceSubType(0x06);
+    pub const MEDIA_PIWG_FIRMWARE_FILE: DeviceSubType = DeviceSubType(6);
     /// PIWG Firmware Volume.
-    pub const MEDIA_PIWG_FIRMWARE_VOLUME: DeviceSubType = DeviceSubType(0x07);
+    pub const MEDIA_PIWG_FIRMWARE_VOLUME: DeviceSubType = DeviceSubType(7);
     /// Relative Offset Range.
-    pub const MEDIA_RELATIVE_OFFSET_RANGE: DeviceSubType = DeviceSubType(0x08);
+    pub const MEDIA_RELATIVE_OFFSET_RANGE: DeviceSubType = DeviceSubType(8);
     /// RAM Disk Device Path.
-    pub const MEDIA_RAM_DISK: DeviceSubType = DeviceSubType(0x09);
+    pub const MEDIA_RAM_DISK: DeviceSubType = DeviceSubType(9);
 
     /// BIOS Boot Specification Device Path.
-    pub const BIOS_BOOT_SPECIFICATION: DeviceSubType = DeviceSubType(0x01);
+    pub const BIOS_BOOT_SPECIFICATION: DeviceSubType = DeviceSubType(1);
 
     /// End this instance of a Device Path and start a new one.
     pub const END_INSTANCE: DeviceSubType = DeviceSubType(0x01);
@@ -541,49 +601,21 @@ impl DeviceSubType {
     pub const END_ENTIRE: DeviceSubType = DeviceSubType(0xff);
 }
 
-/// ACPI Device Path
-#[repr(C, packed)]
-pub struct AcpiDevicePath {
-    header: DevicePathHeader,
+/// Error returned when converting from a [`DevicePathNode`] to a more
+/// specific node type.
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+pub enum NodeConversionError {
+    /// The length of the node data is not valid for its type.
+    InvalidLength,
 
-    /// Device's PnP hardware ID stored in a numeric 32-bit compressed EISA-type ID. This value must match the
-    /// corresponding _HID in the ACPI name space.
-    pub hid: u32,
-    /// Unique ID that is required by ACPI if two devices have the same _HID. This value must also match the
-    /// corresponding _UID/_HID pair in the ACPI name space. Only the 32-bit numeric value type of _UID is supported;
-    /// thus strings must not be used for the _UID in the ACPI name space.
-    pub uid: u32,
-}
-
-/// File Path Media Device Path.
-#[repr(C, packed)]
-pub struct FilePathMediaDevicePath {
-    header: DevicePathHeader,
-    path_name: [u16],
-}
-
-impl FilePathMediaDevicePath {
-    /// Get the path. An [`UnalignedCStr16`] is returned since this is a
-    /// packed struct.
-    pub fn path_name(&self) -> UnalignedCStr16<'_> {
-        // Safety: creating this `UnalignedCStr16` is safe because the
-        // `path_name` pointer is valid (although potentially
-        // unaligned), and the lifetime of the output is tied to `self`,
-        // so there's no possibility of use-after-free.
-        unsafe {
-            // Use `addr_of` to avoid creating an unaligned reference.
-            let ptr: *const [u16] = ptr::addr_of!(self.path_name);
-            let (ptr, len): (*const (), usize) = ptr.to_raw_parts();
-            UnalignedCStr16::new(self, ptr.cast::<u16>(), len)
-        }
-    }
+    /// The node type is not currently supported.
+    UnsupportedType,
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::CString16;
-    use alloc_api::vec::Vec;
+    use alloc::vec::Vec;
 
     /// Create a node to `path` from raw data.
     fn add_node(path: &mut Vec<u8>, device_type: u8, sub_type: u8, node_data: &[u8]) {
@@ -623,7 +655,7 @@ mod tests {
         raw_data
     }
 
-    /// Check that `node` has the expected content.    
+    /// Check that `node` has the expected content.
     fn check_node(node: &DevicePathNode, device_type: u8, sub_type: u8, node_data: &[u8]) {
         assert_eq!(node.device_type().0, device_type);
         assert_eq!(node.sub_type().0, sub_type);
@@ -688,32 +720,4 @@ mod tests {
         // Only two instances.
         assert!(iter.next().is_none());
     }
-
-    #[test]
-    fn test_file_path_media() {
-        // Manually create data for a `FilePathMediaDevicePath` node.
-        let raw_data: [u16; 7] = [
-            // MEDIA | MEDIA_FILE_PATH
-            0x0404,
-            // Length
-            0x00_0e,
-            b't'.into(),
-            b'e'.into(),
-            b's'.into(),
-            b't'.into(),
-            // Trailing null.
-            0,
-        ];
-
-        // Convert the raw data to a `DevicePath` node.
-        let dp = unsafe { DevicePathNode::from_ffi_ptr(raw_data.as_ptr().cast()) };
-        assert_eq!(dp.length(), 14);
-
-        // Check that the `file_name` is correct.
-        let fpm = dp.as_file_path_media_device_path().unwrap();
-        assert_eq!(
-            fpm.path_name().to_cstring16().unwrap(),
-            CString16::try_from("test").unwrap()
-        );
-    }
 }
