@@ -3,16 +3,16 @@
 use crate::{
     data_types::FromSliceWithNulError,
     proto::device_path::{DevicePath, FfiDevicePath},
-    proto::Protocol,
+    proto::unsafe_protocol,
     table::boot::MemoryType,
-    unsafe_guid, CStr16, Handle, Status,
+    util::usize_from_u32,
+    CStr16, Handle, Status,
 };
 use core::{ffi::c_void, mem, slice};
 
 /// The LoadedImage protocol. This can be opened on any image handle using the `HandleProtocol` boot service.
 #[repr(C)]
-#[unsafe_guid("5b1b31a1-9562-11d2-8e3f-00a0c969723b")]
-#[derive(Protocol)]
+#[unsafe_protocol("5b1b31a1-9562-11d2-8e3f-00a0c969723b")]
 pub struct LoadedImage {
     revision: u32,
     parent_handle: Handle,
@@ -52,7 +52,8 @@ pub enum LoadOptionsError {
 
 impl LoadedImage {
     /// Returns a handle to the storage device on which the image is located.
-    pub fn device(&self) -> Handle {
+    #[must_use]
+    pub const fn device(&self) -> Handle {
         self.device_handle
     }
 
@@ -60,6 +61,7 @@ impl LoadedImage {
     ///
     /// Return `None` if the pointer to the file path portion specific to
     /// DeviceHandle that the EFI Image was loaded from is null.
+    #[must_use]
     pub fn file_path(&self) -> Option<&DevicePath> {
         if self.file_path.is_null() {
             None
@@ -77,7 +79,7 @@ impl LoadedImage {
     /// [`&CStr16`]: `CStr16`
     /// [`load_options_as_bytes`]: `Self::load_options_as_bytes`
     pub fn load_options_as_cstr16(&self) -> Result<&CStr16, LoadOptionsError> {
-        let load_options_size = usize::try_from(self.load_options_size).unwrap();
+        let load_options_size = usize_from_u32(self.load_options_size);
 
         if self.load_options.is_null() {
             Err(LoadOptionsError::NotSet)
@@ -106,6 +108,7 @@ impl LoadedImage {
     /// Returns `None` if load options are not set.
     ///
     /// [`load_options_as_cstr16`]: `Self::load_options_as_cstr16`
+    #[must_use]
     pub fn load_options_as_bytes(&self) -> Option<&[u8]> {
         if self.load_options.is_null() {
             None
@@ -113,7 +116,7 @@ impl LoadedImage {
             unsafe {
                 Some(slice::from_raw_parts(
                     self.load_options,
-                    usize::try_from(self.load_options_size).unwrap(),
+                    usize_from_u32(self.load_options_size),
                 ))
             }
         }
@@ -164,7 +167,8 @@ impl LoadedImage {
     }
 
     /// Returns the base address and the size in bytes of the loaded image.
-    pub fn info(&self) -> (*const c_void, u64) {
+    #[must_use]
+    pub const fn info(&self) -> (*const c_void, u64) {
         (self.image_base, self.image_size)
     }
 }
