@@ -16,12 +16,10 @@ use core::ffi::c_void;
 use core::fmt::Debug;
 use core::mem;
 use core::ptr;
-#[cfg(feature = "exts")]
-use {
-    crate::ResultExt,
-    alloc_api::{alloc, alloc::Layout, boxed::Box},
-    core::slice,
-};
+#[cfg(all(feature = "unstable", feature = "alloc"))]
+use {alloc::alloc::Global, core::alloc::Allocator};
+#[cfg(feature = "alloc")]
+use {alloc::boxed::Box, uefi::mem::make_boxed};
 
 pub use self::info::{FileInfo, FileProtocolInfo, FileSystemInfo, FileSystemVolumeLabel, FromUefi};
 pub use self::{dir::Directory, regular::RegularFile};
@@ -43,16 +41,24 @@ pub trait File: Sized {
     /// * `attributes`  Only valid when `FILE_MODE_CREATE` is used as a mode
     ///
     /// # Errors
-    /// * `uefi::Status::INVALID_PARAMETER`  The filename exceeds the maximum length of 255 chars
-    /// * `uefi::Status::NOT_FOUND`          Could not find file
-    /// * `uefi::Status::NO_MEDIA`           The device has no media
-    /// * `uefi::Status::MEDIA_CHANGED`      The device has a different medium in it
-    /// * `uefi::Status::DEVICE_ERROR`       The device reported an error
-    /// * `uefi::Status::VOLUME_CORRUPTED`   The filesystem structures are corrupted
-    /// * `uefi::Status::WRITE_PROTECTED`    Write/Create attempted on readonly file
-    /// * `uefi::Status::ACCESS_DENIED`      The service denied access to the file
-    /// * `uefi::Status::OUT_OF_RESOURCES`    Not enough resources to open file
-    /// * `uefi::Status::VOLUME_FULL`        The volume is full
+    ///
+    /// See section `EFI_FILE_PROTOCOL.Open()` in the UEFI Specification for more details.
+    /// Note that [`INVALID_PARAMETER`] is not listed in the specification as one of the
+    /// errors returned by this function, but some implementations (such as EDK2) perform
+    /// additional validation and may return that status for invalid inputs.
+    ///
+    /// [`INVALID_PARAMETER`]: uefi::Status::INVALID_PARAMETER
+    ///
+    /// * [`uefi::Status::INVALID_PARAMETER`]
+    /// * [`uefi::Status::NOT_FOUND`]
+    /// * [`uefi::Status::NO_MEDIA`]
+    /// * [`uefi::Status::MEDIA_CHANGED`]
+    /// * [`uefi::Status::DEVICE_ERROR`]
+    /// * [`uefi::Status::VOLUME_CORRUPTED`]
+    /// * [`uefi::Status::WRITE_PROTECTED`]
+    /// * [`uefi::Status::ACCESS_DENIED`]
+    /// * [`uefi::Status::OUT_OF_RESOURCES`]
+    /// * [`uefi::Status::VOLUME_FULL`]
     fn open(
         &mut self,
         filename: &CStr16,
@@ -79,7 +85,10 @@ pub trait File: Sized {
     /// Closes and deletes this file
     ///
     /// # Warnings
-    /// * `uefi::Status::WARN_DELETE_FAILURE` The file was closed, but deletion failed
+    ///
+    /// See section `EFI_FILE_PROTOCOL.Delete()` in the UEFI Specification for more details.
+    ///
+    /// * [`uefi::Status::WARN_DELETE_FAILURE`]
     fn delete(mut self) -> Result {
         let result = (self.imp().delete)(self.imp()).into();
         mem::forget(self);
@@ -97,11 +106,14 @@ pub trait File: Sized {
     /// * `buffer`  Buffer that the information should be written into
     ///
     /// # Errors
-    /// * `uefi::Status::UNSUPPORTED`        The file does not possess this information type
-    /// * `uefi::Status::NO_MEDIA`           The device has no medium
-    /// * `uefi::Status::DEVICE_ERROR`       The device reported an error
-    /// * `uefi::Status::VOLUME_CORRUPTED`   The file system structures are corrupted
-    /// * `uefi::Status::BUFFER_TOO_SMALL`   The buffer is too small for the requested
+    ///
+    /// See section `EFI_FILE_PROTOCOL.GetInfo()` in the UEFI Specification for more details.
+    ///
+    /// * [`uefi::Status::UNSUPPORTED`]
+    /// * [`uefi::Status::NO_MEDIA`]
+    /// * [`uefi::Status::DEVICE_ERROR`]
+    /// * [`uefi::Status::VOLUME_CORRUPTED`]
+    /// * [`uefi::Status::BUFFER_TOO_SMALL`]
     fn get_info<'buf, Info: FileProtocolInfo + ?Sized>(
         &mut self,
         buffer: &'buf mut [u8],
@@ -139,84 +151,75 @@ pub trait File: Sized {
     /// * `info`  Info that should be set for the file
     ///
     /// # Errors
-    /// * `uefi::Status::UNSUPPORTED`       The file does not possess this information type
-    /// * `uefi::Status::NO_MEDIA`          The device has no medium
-    /// * `uefi::Status::DEVICE_ERROR`      The device reported an error
-    /// * `uefi::Status::VOLUME_CORRUPTED`  The file system structures are corrupted
-    /// * `uefi::Status::WRITE_PROTECTED`   Attempted to set information on a read-only media
-    /// * `uefi::Status::ACCESS_DENIED`     Requested change is invalid for this information type
-    /// * `uefi::Status::VOLUME_FULL`       Not enough space left on the volume to change the info
+    ///
+    /// See section `EFI_FILE_PROTOCOL.SetInfo()` in the UEFI Specification for more details.
+    ///
+    /// * [`uefi::Status::UNSUPPORTED`]
+    /// * [`uefi::Status::NO_MEDIA`]
+    /// * [`uefi::Status::DEVICE_ERROR`]
+    /// * [`uefi::Status::VOLUME_CORRUPTED`]
+    /// * [`uefi::Status::WRITE_PROTECTED`]
+    /// * [`uefi::Status::ACCESS_DENIED`]
+    /// * [`uefi::Status::VOLUME_FULL`]
+    /// * [`uefi::Status::BAD_BUFFER_SIZE`]
     fn set_info<Info: FileProtocolInfo + ?Sized>(&mut self, info: &Info) -> Result {
         let info_ptr = (info as *const Info).cast::<c_void>();
-        let info_size = mem::size_of_val(&info);
+        let info_size = mem::size_of_val(info);
         unsafe { (self.imp().set_info)(self.imp(), &Info::GUID, info_size, info_ptr).into() }
     }
 
     /// Flushes all modified data associated with the file handle to the device
     ///
     /// # Errors
-    /// * `uefi::Status::NO_MEDIA`           The device has no media
-    /// * `uefi::Status::DEVICE_ERROR`       The device reported an error
-    /// * `uefi::Status::VOLUME_CORRUPTED`   The filesystem structures are corrupted
-    /// * `uefi::Status::WRITE_PROTECTED`    The file or medium is write protected
-    /// * `uefi::Status::ACCESS_DENIED`      The file was opened read only
-    /// * `uefi::Status::VOLUME_FULL`        The volume is full
+    ///
+    /// See section `EFI_FILE_PROTOCOL.Flush()` in the UEFI Specification for more details.
+    ///
+    /// * [`uefi::Status::NO_MEDIA`]
+    /// * [`uefi::Status::DEVICE_ERROR`]
+    /// * [`uefi::Status::VOLUME_CORRUPTED`]
+    /// * [`uefi::Status::WRITE_PROTECTED`]
+    /// * [`uefi::Status::ACCESS_DENIED`]
+    /// * [`uefi::Status::VOLUME_FULL`]
     fn flush(&mut self) -> Result {
         (self.imp().flush)(self.imp()).into()
     }
 
-    #[cfg(feature = "exts")]
-    /// Get the dynamically allocated info for a file
+    /// Read the dynamically allocated info for a file.
+    #[cfg(feature = "alloc")]
     fn get_boxed_info<Info: FileProtocolInfo + ?Sized + Debug>(&mut self) -> Result<Box<Info>> {
-        // Initially try get_info with an empty array, this should always fail
-        // as all Info types at least need room for a null-terminator.
-        let size = match self
-            .get_info::<Info>(&mut [])
-            .expect_err("zero sized get_info unexpectedly succeeded")
-            .split()
-        {
-            (s, None) => return Err(s.into()),
-            (_, Some(size)) => size,
-        };
-
-        // We add trailing padding because the size of a rust structure must
-        // always be a multiple of alignment.
-        let layout = Layout::from_size_align(size, Info::alignment())
-            .unwrap()
-            .pad_to_align();
-
-        // Allocate the buffer.
-        let data: *mut u8 = unsafe {
-            let data = alloc::alloc(layout);
-            if data.is_null() {
-                return Err(Status::OUT_OF_RESOURCES.into());
-            }
-            data
-        };
-
-        // Get the file info using the allocated buffer for storage.
-        let info = {
-            let buffer = unsafe { slice::from_raw_parts_mut(data, layout.size()) };
-            self.get_info::<Info>(buffer).discard_errdata()
-        };
-
-        // If an error occurred, deallocate the memory before returning.
-        let info = match info {
-            Ok(info) => info,
-            Err(err) => {
-                unsafe { alloc::dealloc(data, layout) };
-                return Err(err);
-            }
-        };
-
-        // Wrap the file info in a box so that it will be deallocated on
-        // drop. This is valid because the memory was allocated with the
-        // global allocator.
-        unsafe { Ok(Box::from_raw(info)) }
+        let fetch_data_fn = |buf| self.get_info::<Info>(buf);
+        #[cfg(not(feature = "unstable"))]
+        let file_info = make_boxed::<Info, _>(fetch_data_fn)?;
+        #[cfg(feature = "unstable")]
+        let file_info = make_boxed::<Info, _, _>(fetch_data_fn, Global)?;
+        Ok(file_info)
     }
+
+    /// Read the dynamically allocated info for a file.
+    #[cfg(all(feature = "unstable", feature = "alloc"))]
+    fn get_boxed_info_in<Info: FileProtocolInfo + ?Sized + Debug, A: Allocator>(
+        &mut self,
+        allocator: A,
+    ) -> Result<Box<Info>> {
+        let fetch_data_fn = |buf| self.get_info::<Info>(buf);
+        let file_info = make_boxed::<Info, _, A>(fetch_data_fn, allocator)?;
+        Ok(file_info)
+    }
+
+    /// Returns if the underlying file is a regular file.
+    /// The result is an error if the underlying file was already closed or deleted.
+    ///
+    /// UEFI file system protocol only knows "regular files" and "directories".
+    fn is_regular_file(&self) -> Result<bool>;
+
+    /// Returns if the underlying file is a directory.
+    /// The result is an error if the underlying file was already closed or deleted.
+    ///
+    /// UEFI file system protocol only knows "regular files" and "directories".
+    fn is_directory(&self) -> Result<bool>;
 }
 
-// Internal File helper methods to access the funciton pointer table.
+// Internal File helper methods to access the function pointer table.
 trait FileInternal: File {
     fn imp(&mut self) -> &mut FileImpl {
         unsafe { &mut *self.handle().0 }
@@ -233,29 +236,31 @@ impl<T: File> FileInternal for T {}
 ///
 /// Dropping this structure will result in the file handle being closed.
 #[repr(transparent)]
+#[derive(Debug)]
 pub struct FileHandle(*mut FileImpl);
 
 impl FileHandle {
-    pub(super) unsafe fn new(ptr: *mut FileImpl) -> Self {
+    pub(super) const unsafe fn new(ptr: *mut FileImpl) -> Self {
         Self(ptr)
     }
 
     /// Converts `File` into a more specific subtype based on if it is a
-    /// directory or not. It does this via a call to `get_position`.
-    pub fn into_type(mut self) -> Result<FileType> {
+    /// directory or not. Wrapper around [Self::is_regular_file].
+    pub fn into_type(self) -> Result<FileType> {
         use FileType::*;
 
-        // get_position fails with EFI_UNSUPPORTED on directories
-        let mut pos = 0;
-        match (self.imp().get_position)(self.imp(), &mut pos) {
-            Status::SUCCESS => unsafe { Ok(Regular(RegularFile::new(self))) },
-            Status::UNSUPPORTED => unsafe { Ok(Dir(Directory::new(self))) },
-            s => Err(s.into()),
-        }
+        self.is_regular_file().map(|is_file| {
+            if is_file {
+                unsafe { Regular(RegularFile::new(self)) }
+            } else {
+                unsafe { Dir(Directory::new(self)) }
+            }
+        })
     }
 
     /// If the handle represents a directory, convert it into a
     /// [`Directory`]. Otherwise returns `None`.
+    #[must_use]
     pub fn into_directory(self) -> Option<Directory> {
         if let Ok(FileType::Dir(dir)) = self.into_type() {
             Some(dir)
@@ -266,6 +271,7 @@ impl FileHandle {
 
     /// If the handle represents a regular file, convert it into a
     /// [`RegularFile`]. Otherwise returns `None`.
+    #[must_use]
     pub fn into_regular_file(self) -> Option<RegularFile> {
         if let Ok(FileType::Regular(regular)) = self.into_type() {
             Some(regular)
@@ -280,6 +286,23 @@ impl File for FileHandle {
     fn handle(&mut self) -> &mut FileHandle {
         self
     }
+
+    fn is_regular_file(&self) -> Result<bool> {
+        let this = unsafe { self.0.as_mut().unwrap() };
+
+        // - get_position fails with EFI_UNSUPPORTED on directories
+        // - result is an error if the underlying file was already closed or deleted.
+        let mut pos = 0;
+        match (this.get_position)(this, &mut pos) {
+            Status::SUCCESS => Ok(true),
+            Status::UNSUPPORTED => Ok(false),
+            s => Err(s.into()),
+        }
+    }
+
+    fn is_directory(&self) -> Result<bool> {
+        self.is_regular_file().map(|b| !b)
+    }
 }
 
 impl Drop for FileHandle {
@@ -303,6 +326,19 @@ pub(super) struct FileImpl {
     ) -> Status,
     close: extern "efiapi" fn(this: &mut FileImpl) -> Status,
     delete: extern "efiapi" fn(this: &mut FileImpl) -> Status,
+    /// # Read from Regular Files
+    /// If `self` is not a directory, the function reads the requested number of bytes from the file
+    /// at the file’s current position and returns them in `buffer`. If the read goes beyond the end
+    /// of the file, the read length is truncated to the end of the file. The file’s current
+    /// position is increased by the number of bytes returned.
+    ///
+    /// # Read from Directory
+    /// If `self` is a directory, the function reads the directory entry at the file’s current
+    /// position and returns the entry in `buffer`. If the `buffer` is not large enough to hold the
+    /// current directory entry, then `EFI_BUFFER_TOO_SMALL` is returned and the current file
+    /// position is not updated. `buffer_size` is set to be the size of the buffer needed to read
+    /// the entry. On success, the current position is updated to the next directory entry. If there
+    /// are no more directory entries, the read returns a zero-length buffer.
     read: unsafe extern "efiapi" fn(
         this: &mut FileImpl,
         buffer_size: &mut usize,
@@ -330,7 +366,8 @@ pub(super) struct FileImpl {
     flush: extern "efiapi" fn(this: &mut FileImpl) -> Status,
 }
 
-/// Disambiguates the file type. Returned by `File::into_type()`.
+/// Disambiguate the file type. Returned by `File::into_type()`.
+#[derive(Debug)]
 pub enum FileType {
     /// The file was a regular (data) file.
     Regular(RegularFile),
@@ -379,7 +416,7 @@ mod tests {
     use super::*;
     use crate::table::runtime::Time;
     use crate::{CString16, Identify};
-    use alloc_api::vec;
+    use ::alloc::vec;
 
     // Test `get_boxed_info` by setting up a fake file, which is mostly
     // just function pointers. Most of the functions can be empty, only
