@@ -3,6 +3,7 @@
 //! These protocols can be used to interact with network resources.
 
 pub mod pxe;
+pub mod snp;
 
 /// Represents an IPv4/v6 address.
 ///
@@ -13,6 +14,7 @@ pub struct IpAddress(pub [u8; 16]);
 
 impl IpAddress {
     /// Construct a new IPv4 address.
+    #[must_use]
     pub const fn new_v4(ip_addr: [u8; 4]) -> Self {
         let mut buffer = [0; 16];
         buffer[0] = ip_addr[0];
@@ -23,6 +25,7 @@ impl IpAddress {
     }
 
     /// Construct a new IPv6 address.
+    #[must_use]
     pub const fn new_v6(ip_addr: [u8; 16]) -> Self {
         Self(ip_addr)
     }
