@@ -3,11 +3,15 @@
 use core::{
     ffi::c_void,
     iter::from_fn,
+    marker::{PhantomData, PhantomPinned},
+    mem::MaybeUninit,
     ptr::{null, null_mut},
 };
 
+use crate::util::ptr_write_unaligned_and_add;
+use crate::{polyfill::maybe_uninit_slice_as_mut_ptr, proto::unsafe_protocol};
 use bitflags::bitflags;
-use uefi_macros::{unsafe_guid, Protocol};
+use ptr_meta::Pointee;
 
 use crate::{CStr8, Char8, Result, Status};
 
@@ -15,8 +19,7 @@ use super::{IpAddress, MacAddress};
 
 /// PXE Base Code protocol
 #[repr(C)]
-#[unsafe_guid("03c4e603-ac28-11d3-9a2d-0090273fc14d")]
-#[derive(Protocol)]
+#[unsafe_protocol("03c4e603-ac28-11d3-9a2d-0090273fc14d")]
 #[allow(clippy::type_complexity)]
 pub struct BaseCode {
     revision: u64,
@@ -28,7 +31,7 @@ pub struct BaseCode {
         ty: BootstrapType,
         layer: &mut u16,
         use_bis: bool,
-        info: Option<*const DiscoverInfo<[Server; 0]>>,
+        info: *const FfiDiscoverInfo,
     ) -> Status,
     mtftp: unsafe extern "efiapi" fn(
         this: &Self,
@@ -128,16 +131,16 @@ impl BaseCode {
         ty: BootstrapType,
         layer: &mut u16,
         use_bis: bool,
-        info: Option<&DiscoverInfo<[Server]>>,
+        info: Option<&DiscoverInfo>,
     ) -> Result {
-        (self.discover)(
-            self,
-            ty,
-            layer,
-            use_bis,
-            info.map(|info| (info as *const DiscoverInfo<[Server]>).cast()),
-        )
-        .into()
+        let info: *const FfiDiscoverInfo = info
+            .map(|info| {
+                let info_ptr: *const DiscoverInfo = info;
+                info_ptr.cast()
+            })
+            .unwrap_or(null());
+
+        (self.discover)(self, ty, layer, use_bis, info).into()
     }
 
     /// Returns the size of a file located on a TFTP server.
@@ -611,7 +614,8 @@ impl BaseCode {
     }
 
     /// Returns a reference to the `Mode` struct.
-    pub fn mode(&self) -> &Mode {
+    #[must_use]
+    pub const fn mode(&self) -> &Mode {
         unsafe { &*self.mode }
     }
 }
@@ -648,79 +652,116 @@ pub enum BootstrapType {
     PxeTest = 65535,
 }
 
+/// Opaque type that should be used to represent a pointer to a [`DiscoverInfo`] in
+/// foreign function interfaces. This type produces a thin pointer, unlike
+/// [`DiscoverInfo`].
+#[repr(C, packed)]
+pub struct FfiDiscoverInfo {
+    // This representation is recommended by the nomicon:
+    // https://doc.rust-lang.org/stable/nomicon/ffi.html#representing-opaque-structs
+    _data: [u8; 0],
+    _marker: PhantomData<(*mut u8, PhantomPinned)>,
+}
+
 /// This struct contains optional parameters for [`BaseCode::discover`].
 ///
 /// Corresponds to the `EFI_PXE_BASE_CODE_DISCOVER_INFO` type in the C API.
 #[repr(C)]
-pub struct DiscoverInfo<T: ?Sized> {
+#[derive(Pointee)]
+pub struct DiscoverInfo {
     use_m_cast: bool,
     use_b_cast: bool,
     use_u_cast: bool,
     must_use_list: bool,
     server_m_cast_ip: IpAddress,
     ip_cnt: u16,
-    srv_list: T,
+    srv_list: [Server],
 }
 
-impl<const N: usize> DiscoverInfo<[Server; N]> {
+impl DiscoverInfo {
     /// Create a `DiscoverInfo`.
-    pub const fn new(
+    pub fn new_in_buffer<'buf>(
+        buffer: &'buf mut [MaybeUninit<u8>],
         use_m_cast: bool,
         use_b_cast: bool,
         use_u_cast: bool,
         must_use_list: bool,
         server_m_cast_ip: IpAddress,
-        srv_list: [Server; N],
-    ) -> Self {
-        assert!(N <= u16::MAX as usize, "too many servers");
-        let ip_cnt = N as u16;
-        Self {
-            use_m_cast,
-            use_b_cast,
-            use_u_cast,
-            must_use_list,
-            server_m_cast_ip,
-            ip_cnt,
-            srv_list,
+        srv_list: &[Server],
+    ) -> Result<&'buf mut Self> {
+        let server_count = srv_list.len();
+        assert!(server_count <= u16::MAX as usize, "too many servers");
+
+        let required_size = core::mem::size_of::<bool>() * 4
+            + core::mem::size_of::<IpAddress>()
+            + core::mem::size_of::<u16>()
+            + core::mem::size_of::<Server>() * server_count;
+
+        if buffer.len() < required_size {
+            return Err(Status::BUFFER_TOO_SMALL.into());
+        }
+
+        let mut ptr: *mut u8 = maybe_uninit_slice_as_mut_ptr(buffer);
+        unsafe {
+            ptr_write_unaligned_and_add(&mut ptr, use_m_cast);
+            ptr_write_unaligned_and_add(&mut ptr, use_b_cast);
+            ptr_write_unaligned_and_add(&mut ptr, use_u_cast);
+            ptr_write_unaligned_and_add(&mut ptr, must_use_list);
+            ptr_write_unaligned_and_add(&mut ptr, server_m_cast_ip);
+            ptr_write_unaligned_and_add(&mut ptr, server_count as u16);
+
+            ptr = ptr.add(2); // Align server list (4-byte alignment).
+            core::ptr::copy(srv_list.as_ptr(), ptr.cast(), server_count);
+
+            let ptr: *mut Self =
+                ptr_meta::from_raw_parts_mut(buffer.as_mut_ptr().cast(), server_count);
+            Ok(&mut *ptr)
         }
     }
 }
 
-impl<T> DiscoverInfo<T> {
+impl DiscoverInfo {
     /// Returns whether discovery should use multicast.
-    pub fn use_m_cast(&self) -> bool {
+    #[must_use]
+    pub const fn use_m_cast(&self) -> bool {
         self.use_m_cast
     }
 
     /// Returns whether discovery should use broadcast.
-    pub fn use_b_cast(&self) -> bool {
+    #[must_use]
+    pub const fn use_b_cast(&self) -> bool {
         self.use_b_cast
     }
 
     /// Returns whether discovery should use unicast.
-    pub fn use_u_cast(&self) -> bool {
+    #[must_use]
+    pub const fn use_u_cast(&self) -> bool {
         self.use_u_cast
     }
 
     /// Returns whether discovery should only accept boot servers in the server
     /// list (boot server verification).
-    pub fn must_use_list(&self) -> bool {
+    #[must_use]
+    pub const fn must_use_list(&self) -> bool {
         self.must_use_list
     }
 
     /// Returns the address used in multicast discovery.
-    pub fn server_m_cast_ip(&self) -> &IpAddress {
+    #[must_use]
+    pub const fn server_m_cast_ip(&self) -> &IpAddress {
         &self.server_m_cast_ip
     }
 
     /// Returns the amount of Boot Server.
-    pub fn ip_cnt(&self) -> u16 {
+    #[must_use]
+    pub const fn ip_cnt(&self) -> u16 {
         self.ip_cnt
     }
 
     /// Returns the Boot Server list used for unicast discovery or boot server
     /// verification.
-    pub fn srv_list(&self) -> &T {
+    #[must_use]
+    pub const fn srv_list(&self) -> &[Server] {
         &self.srv_list
     }
 }
@@ -742,6 +783,7 @@ impl Server {
     /// Construct a `Server` for a Boot Server reply type. If `ip_addr` is not
     /// `None` only Boot Server replies with matching the IP address will be
     /// accepted.
+    #[must_use]
     pub fn new(ty: u16, ip_addr: Option<IpAddress>) -> Self {
         Self {
             ty,
@@ -753,7 +795,8 @@ impl Server {
 
     /// Returns a `None` if the any response should be accepted or the IP
     /// address of a Boot Server whose responses should be accepted.
-    pub fn ip_addr(&self) -> Option<&IpAddress> {
+    #[must_use]
+    pub const fn ip_addr(&self) -> Option<&IpAddress> {
         if self.accept_any_response {
             None
         } else {
@@ -837,6 +880,7 @@ impl IpFilter {
     /// # Panics
     ///
     /// Panics if `ip_list` contains more than 8 entries.
+    #[must_use]
     pub fn new(filters: IpFilters, ip_list: &[IpAddress]) -> Self {
         assert!(ip_list.len() <= 8);
 
@@ -854,6 +898,7 @@ impl IpFilter {
 
     /// A list of IP addresses other than the Station Ip that should be
     /// enabled. Maybe be multicast or unicast.
+    #[must_use]
     pub fn ip_list(&self) -> &[IpAddress] {
         &self.ip_list[..usize::from(self.ip_cnt)]
     }
@@ -944,22 +989,26 @@ impl DhcpV4Packet {
     pub const DHCP_MAGIK: u32 = 0x63825363;
 
     /// Transaction ID, a random number, used to match this boot request with the responses it generates.
-    pub fn bootp_ident(&self) -> u32 {
+    #[must_use]
+    pub const fn bootp_ident(&self) -> u32 {
         u32::from_be(self.bootp_ident)
     }
 
     /// Filled in by client, seconds elapsed since client started trying to boot.
-    pub fn bootp_seconds(&self) -> u16 {
+    #[must_use]
+    pub const fn bootp_seconds(&self) -> u16 {
         u16::from_be(self.bootp_seconds)
     }
 
     /// The flags.
-    pub fn bootp_flags(&self) -> DhcpV4Flags {
+    #[must_use]
+    pub const fn bootp_flags(&self) -> DhcpV4Flags {
         DhcpV4Flags::from_bits_truncate(u16::from_be(self.bootp_flags))
     }
 
     /// A magic cookie, should be [`Self::DHCP_MAGIK`].
-    pub fn dhcp_magik(&self) -> u32 {
+    #[must_use]
+    pub const fn dhcp_magik(&self) -> u32 {
         u32::from_be(self.dhcp_magik)
     }
 }
@@ -988,6 +1037,7 @@ pub struct DhcpV6Packet {
 
 impl DhcpV6Packet {
     /// The transaction id.
+    #[must_use]
     pub fn transaction_id(&self) -> u32 {
         u32::from(self.transaction_id[0]) << 16
             | u32::from(self.transaction_id[1]) << 8
