@@ -1,9 +1,5 @@
 use super::{Error, Result};
-use core::{
-    convert::Infallible,
-    ops::{ControlFlow, FromResidual, Try},
-};
-use core::{fmt::Debug, num::NonZeroUsize};
+use core::fmt::Debug;
 
 /// Bit indicating that an UEFI status code is an error
 const ERROR_BIT: usize = 1 << (core::mem::size_of::<usize>() * 8 - 1);
@@ -16,6 +12,12 @@ newtype_enum! {
 /// enum, as injecting an unknown value in a Rust enum is undefined behaviour.
 ///
 /// For lack of a better option, we therefore model them as a newtype of usize.
+///
+/// For a convenient integration into the Rust ecosystem, there are multiple
+/// factory methods to convert a Status into a [`uefi::Result`]:
+/// - [`Status::into_with`]
+/// - [`Status::into_with_val`]
+/// - [`Status::into_with_err`]
 #[must_use]
 pub enum Status: usize => {
     /// The operation completed successfully.
@@ -110,23 +112,29 @@ pub enum Status: usize => {
 impl Status {
     /// Returns true if status code indicates success.
     #[inline]
+    #[must_use]
     pub fn is_success(self) -> bool {
         self == Status::SUCCESS
     }
 
     /// Returns true if status code indicates a warning.
     #[inline]
+    #[must_use]
     pub fn is_warning(self) -> bool {
         (self != Status::SUCCESS) && (self.0 & ERROR_BIT == 0)
     }
 
     /// Returns true if the status code indicates an error.
     #[inline]
-    pub fn is_error(self) -> bool {
+    #[must_use]
+    pub const fn is_error(self) -> bool {
         self.0 & ERROR_BIT != 0
     }
 
-    /// Converts this status code into a result with a given value.
+    /// Converts this status code into a [`uefi::Result`] with a given `Ok` value.
+    ///
+    /// If the status does not indicate success, the status representing the specific error
+    /// code is embedded into the `Err` variant of type [`uefi::Error`].
     #[inline]
     pub fn into_with_val<T>(self, val: impl FnOnce() -> T) -> Result<T, ()> {
         if self.is_success() {
@@ -136,7 +144,10 @@ impl Status {
         }
     }
 
-    /// Converts this status code into a result with a given error payload
+    /// Converts this status code into a [`uefi::Result`] with a given `Err` payload.
+    ///
+    /// If the status does not indicate success, the status representing the specific error
+    /// code is embedded into the `Err` variant of type [`uefi::Error`].
     #[inline]
     pub fn into_with_err<ErrData: Debug>(
         self,
@@ -149,7 +160,10 @@ impl Status {
         }
     }
 
-    /// Convert this status code into a result with a given value and error payload
+    /// Convert this status code into a result with a given `Ok` value and `Err` payload.
+    ///
+    /// If the status does not indicate success, the status representing the specific error
+    /// code is embedded into the `Err` variant of type [`uefi::Error`].
     #[inline]
     pub fn into_with<T, ErrData: Debug>(
         self,
@@ -172,54 +186,9 @@ impl From<Status> for Result<(), ()> {
     }
 }
 
-pub struct StatusResidual(NonZeroUsize);
-
-impl Try for Status {
-    type Output = ();
-    type Residual = StatusResidual;
-
-    fn branch(self) -> ControlFlow<Self::Residual, Self::Output> {
-        match NonZeroUsize::new(self.0) {
-            Some(r) => ControlFlow::Break(StatusResidual(r)),
-            None => ControlFlow::Continue(()),
-        }
-    }
-
-    fn from_output(_output: Self::Output) -> Self {
-        Self::SUCCESS
-    }
-}
-
-impl FromResidual for Status {
-    fn from_residual(r: StatusResidual) -> Self {
-        Status(r.0.into())
-    }
-}
-
-impl<T> FromResidual<StatusResidual> for Result<T, ()> {
-    fn from_residual(r: StatusResidual) -> Self {
-        Err(Status(r.0.into()).into())
-    }
-}
-
-impl FromResidual<core::result::Result<Infallible, Error>> for Status {
-    fn from_residual(r: core::result::Result<Infallible, Error>) -> Self {
-        match r {
-            Err(err) => err.status(),
-            Ok(infallible) => match infallible {},
-        }
-    }
-}
-
-// FIXME: This conversion will go away along with usage of the ucs2 crate
-
-impl From<ucs2::Error> for Status {
-    fn from(other: ucs2::Error) -> Self {
-        use ucs2::Error;
-        match other {
-            Error::BufferOverflow => Status::BUFFER_TOO_SMALL,
-            Error::MultiByte => Status::UNSUPPORTED,
-        }
+impl core::fmt::Display for Status {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        Debug::fmt(self, f)
     }
 }
 
