@@ -1,16 +1,16 @@
 //! UEFI services available at runtime, even after the OS boots.
 
-use super::Header;
-#[cfg(feature = "exts")]
+use super::{Header, Revision};
+#[cfg(feature = "alloc")]
 use crate::data_types::FromSliceWithNulError;
 use crate::result::Error;
 use crate::table::boot::MemoryDescriptor;
-use crate::{CStr16, Char16, Guid, Result, Status};
-#[cfg(feature = "exts")]
-use alloc_api::{vec, vec::Vec};
+use crate::{guid, CStr16, Char16, Guid, Result, Status};
+#[cfg(feature = "alloc")]
+use alloc::{vec, vec::Vec};
 use bitflags::bitflags;
 use core::fmt::{Debug, Formatter};
-#[cfg(feature = "exts")]
+#[cfg(feature = "alloc")]
 use core::mem;
 use core::mem::MaybeUninit;
 use core::{fmt, ptr};
@@ -157,7 +157,7 @@ impl RuntimeServices {
     }
 
     /// Get the names and vendor GUIDs of all currently-set variables.
-    #[cfg(feature = "exts")]
+    #[cfg(feature = "alloc")]
     pub fn variable_keys(&self) -> Result<Vec<VariableKey>> {
         let mut all_variables = Vec::new();
 
@@ -245,14 +245,26 @@ impl RuntimeServices {
         }
     }
 
+    /// Deletes a UEFI variable.
+    pub fn delete_variable(&self, name: &CStr16, vendor: &VariableVendor) -> Result {
+        self.set_variable(name, vendor, VariableAttributes::empty(), &[])
+    }
+
     /// Get information about UEFI variable storage space for the type
     /// of variable specified in `attributes`.
     ///
+    /// This operation is only supported starting with UEFI 2.0; earlier
+    /// versions will fail with [`Status::UNSUPPORTED`].
+    ///
     /// See [`VariableStorageInfo`] for details of the information returned.
     pub fn query_variable_info(
         &self,
         attributes: VariableAttributes,
     ) -> Result<VariableStorageInfo> {
+        if self.header.revision < Revision::EFI_2_00 {
+            return Err(Status::UNSUPPORTED.into());
+        }
+
         let mut info = VariableStorageInfo::default();
         unsafe {
             (self.query_variable_info)(
@@ -398,7 +410,8 @@ impl Time {
     ///
     /// [`FileInfo`]: uefi::proto::media::file::FileInfo
     /// [`File::set_info`]: uefi::proto::media::file::File::set_info
-    pub fn invalid() -> Self {
+    #[must_use]
+    pub const fn invalid() -> Self {
         Self {
             year: 0,
             month: 0,
@@ -415,6 +428,7 @@ impl Time {
     }
 
     /// True if all fields are within valid ranges, false otherwise.
+    #[must_use]
     pub fn is_valid(&self) -> bool {
         (1900..=9999).contains(&self.year)
             && (1..=12).contains(&self.month)
@@ -428,42 +442,50 @@ impl Time {
     }
 
     /// Query the year.
-    pub fn year(&self) -> u16 {
+    #[must_use]
+    pub const fn year(&self) -> u16 {
         self.year
     }
 
     /// Query the month.
-    pub fn month(&self) -> u8 {
+    #[must_use]
+    pub const fn month(&self) -> u8 {
         self.month
     }
 
     /// Query the day.
-    pub fn day(&self) -> u8 {
+    #[must_use]
+    pub const fn day(&self) -> u8 {
         self.day
     }
 
     /// Query the hour.
-    pub fn hour(&self) -> u8 {
+    #[must_use]
+    pub const fn hour(&self) -> u8 {
         self.hour
     }
 
     /// Query the minute.
-    pub fn minute(&self) -> u8 {
+    #[must_use]
+    pub const fn minute(&self) -> u8 {
         self.minute
     }
 
     /// Query the second.
-    pub fn second(&self) -> u8 {
+    #[must_use]
+    pub const fn second(&self) -> u8 {
         self.second
     }
 
     /// Query the nanosecond.
-    pub fn nanosecond(&self) -> u32 {
+    #[must_use]
+    pub const fn nanosecond(&self) -> u32 {
         self.nanosecond
     }
 
     /// Query the time offset in minutes from UTC, or None if using local time.
-    pub fn time_zone(&self) -> Option<i16> {
+    #[must_use]
+    pub const fn time_zone(&self) -> Option<i16> {
         if self.time_zone == 2047 {
             None
         } else {
@@ -472,7 +494,8 @@ impl Time {
     }
 
     /// Query the daylight savings time information.
-    pub fn daylight(&self) -> Daylight {
+    #[must_use]
+    pub const fn daylight(&self) -> Daylight {
         self.daylight
     }
 }
@@ -512,11 +535,11 @@ impl fmt::Display for Time {
             let fraction_part = offset_in_hours - (integer_part as f32);
             // most time zones
             if fraction_part == 0.0 {
-                write!(f, "UTC+{}", offset_in_hours)?;
+                write!(f, "UTC+{offset_in_hours}")?;
             }
             // time zones with 30min offset (and perhaps other special time zones)
             else {
-                write!(f, "UTC+{:.1}", offset_in_hours)?;
+                write!(f, "UTC+{offset_in_hours:.1}")?;
             }
         }
 
@@ -599,27 +622,15 @@ newtype_enum! {
     /// defines some special values, and vendors will define their own.
     pub enum VariableVendor: Guid => {
         /// Used to access global variables.
-        GLOBAL_VARIABLE = Guid::from_values(
-            0x8be4df61,
-            0x93ca,
-            0x11d2,
-            0xaa0d,
-            0x00e098032b8c,
-        ),
+        GLOBAL_VARIABLE = guid!("8be4df61-93ca-11d2-aa0d-00e098032b8c"),
 
         /// Used to access EFI signature database variables.
-        IMAGE_SECURITY_DATABASE = Guid::from_values(
-            0xd719b2cb,
-            0x3d3a,
-            0x4596,
-            0xa3bc,
-            0xdad00e67656f,
-        ),
+        IMAGE_SECURITY_DATABASE = guid!("d719b2cb-3d3a-4596-a3bc-dad00e67656f"),
     }
 }
 
 /// Unique key for a variable.
-#[cfg(feature = "exts")]
+#[cfg(feature = "alloc")]
 #[derive(Debug)]
 pub struct VariableKey {
     name: Vec<u16>,
@@ -627,7 +638,7 @@ pub struct VariableKey {
     pub vendor: VariableVendor,
 }
 
-#[cfg(feature = "exts")]
+#[cfg(feature = "alloc")]
 impl VariableKey {
     /// Name of the variable.
     pub fn name(&self) -> core::result::Result<&CStr16, FromSliceWithNulError> {
@@ -635,14 +646,14 @@ impl VariableKey {
     }
 }
 
-#[cfg(feature = "exts")]
+#[cfg(feature = "alloc")]
 impl fmt::Display for VariableKey {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "VariableKey {{ name: ")?;
 
         match self.name() {
-            Ok(name) => write!(f, "\"{}\"", name)?,
-            Err(err) => write!(f, "Err({:?})", err)?,
+            Ok(name) => write!(f, "\"{name}\"")?,
+            Err(err) => write!(f, "Err({err:?})")?,
         }
 
         write!(f, ", vendor: ")?;
